;
; ideserv.asm - X1541 pclink handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;---------------------------------------
; Initiate communication (0 is returned)
;
;          1             1
;       __________________________
; C64  | ACCU | SECONDARY ADDRESS |
;      |______|___________________|
;
pclink_secaddr  = next_sector+3
pclink_timer	= current_path
pclink_crc	= current_path+2

pclink_init	.proc
		pha
                .if PCLINK_USECRC=1
                tax
                lda crc8,x
                eor pclink_secaddr
                tax
                lda crc8,x
                sta pclink_crc
                .fi
		lda #%11000111
                tay
                sta pclink_timer+1
		and $dd00
		sta $dd00

lp		lda #63-8-32
		sta $dd02

		lda #63-32
		sta $dd02

                jsr pclink_timeout

                bit $dd00
                bvc lp

		lda #63
		sta $dd02
                pla
		jsr pclink_send
		lda pclink_secaddr
;---------------------------------------
; Send a byte (0 is returned)
;
;     C64 ____RDY RDY_____RDY RDY_____RDY RDY_____RDY RDY____
; ATN         \_____/     \_____/     \_____/     \_____/
;
;     PC  __GO   GO_____GO   GO_____GO   GO_____GO   GO_____
; DATA      \_____/     \_____/     \_____/     \_____/     
;               |     |     |     |     |     |     |     |
;     DATA____ _____ _____ _____ _____ _____ _____ _____ ___
; CLK     ____X_7___X_6___X_5___X_4___X_3___X_2___X_1___X_0_
;
pclink_send     .proc
		ldy #$fe
                sty pclink_timer+1
		sec
                rol
		bcs cs1

cs4		ldy #63-8
v2		bit $dd00
		bmi v1
		sty $dd02
                
                asl
                bcc cs2

cs3		ldy #63-16
v4		bit $dd00
		bpl v3
		sty $dd02

		asl
                bcc cs4
                beq cs5

cs1		ldy #63-8-16
v6		bit $dd00
		bmi v5
		sty $dd02

		asl
		bcs cs3
                
cs2		ldy #63
v8		bit $dd00
		bpl v7
		sty $dd02

		asl
                bcc cs4
                bne cs1
cs5		rts

v1		jsr pclink_timeout
                gne v2

v3		jsr pclink_timeout
                gne v4

v5		jsr pclink_timeout
                gne v6

v7		jsr pclink_timeout
                gne v8
		.pend
;---------------------------------------
; Receive a byte (Z and N flags must be set correctly on exit, Y not touched!)
; C=1 on timeout and exit caller
;
;     C64 _______OK   OK_____OK   OK_____OK   OK_____OK   OK____
; ATN            \_____/     \_____/     \_____/     \_____/
;
;     PC  _____RDY RDY_____RDY RDY_____RDY RDY_____RDY RDY______
; DATA         \_____/     \_____/     \_____/     \_____/     
;               |     |     |     |     |     |     |     |
;     DATA___ _____ _____ _____ _____ _____ _____ _____ ________
; CLK     ___X_7___X_6___X_5___X_4___X_3___X_2___X_1___X_0______
;         
pclink_recieve	.proc
		lda #$fe
                sta pclink_timer+1
                sec
lp      	
		ldx #63-8
v2		bit $dd00
		bmi v1
		stx $dd02

                bvs mn
                asl
                ldx $dd00
                bpl v3
                cpx #$c0

                ldx #63
                stx $dd02

                rol
                bcs lp
                rts

mn		rol

v4		ldx $dd00
		bpl v3
                cpx #$c0

                ldx #63
		stx $dd02

                rol
		bcs lp
		rts

v1		jsr pclink_timeout
		gne v2

v3		jsr pclink_timeout
                gne v4
		.pend
;---------------------------------------
; Load a file
;
;         1            1                2        1   1        2        X     1
;       ___________________________________________                             ___
; C64  | "R" | SECONDARY ADDRESS | BYTES LO/HI | 0 |                           | CLOSE
;      |_____|___________________|_____________|___|                           |___
;   ___                                             ___________________________
; OPEN |                                           | ? | BYTES LO/HI| DATA | 0 |
;   ___|                                           |___|____________|______|___|
;
;          1            1                2         1    1   1        2        X      1    1
;       __________________________________________________                                   ___
; C64  | !"R" | SECONDARY ADDRESS | BYTES LO/HI | CRC | 0 |                                 | CLOSE
;      |______|___________________|_____________|_____|___|                                 |___
;   ___                                                    _________________________________
; OPEN |                                                  | ? | BYTES LO/HI| DATA | CRC | 0 |
;   ___|                                                  |___|____________|______|_____|___|
;
; $00 - OK
; $40 - EOF
; $42 - Too much
; 
pclink_load	.proc
		#xit_init

                lda start_zeropage
                cmp #$4c
                beq outm

		lda #$60
                sta pclink_secaddr
		jsr pclink_send_open
                cmp #0			;readonly?
		bne pclink_filenotf

		.if PCLINK_USECRC=1
                lda #"R"+128
                .else
                lda #"R"
                .fi
                jsr pclink_init
                lda #<$ffff
                jsr pclink_send
                lda #>$ffff
                jsr pclink_send
                .if PCLINK_USECRC=1
                lda #$17		;precalculated crc
                jsr pclink_send
                .fi
                jsr pclink_send
                jsr pclink_recieve
                bne kyp2

                jsr pclink_recieve
                sta next_sector
                jsr pclink_recieve
                sta next_sector+1

                inc next_sector
                bne cs1
                inc next_sector+1
                beq kyp2
cs1		jsr pclink_recieve
		tay
                inc next_sector
                bne cs2
                inc next_sector+1
                beq kyp2
cs2		jsr pclink_recieve
		ldx ActualSecAddr	;sec addr!=0?
		bne jklj
		ldy $c3
		lda $c4
jklj		sty $ae
		sta $af

		jsr OP_load_loading

		ldy #0
eiqj		inc next_sector
                beq vx
vx2		jsr pclink_recieve
		jsr start_zeropage
		iny
		bne eiqj
                jsr pclink_stop
		inc $af
		bne eiqj
outm            sec
                ldy #0
                rts			;out of memory

vx              inc next_sector+1
                bne vx2
		jsr pclink_recieve
                #phy_a
		jsr pclink_send_close
                #ply_a
		clc
		rts			;exit

kyp2		lda #$02		;loaderr
		sta Status
                clc
                rts

pclink_filenotf	ldy #$42
                sty Status
sto		sec
                rts
		.pend

peek_r		.proc
		.logical start_zeropage
		sta cfg_stnd		;3
at1		lda $ff00,x		;3
		.if SUPERCPU=1
                jsr switch_16k		;SCPU bug!
                .else
		sta cfg_open		;3
                .fi
		jmp pclink_send         ;3
		.here
		.pend

pclink_saveerr  .proc
		lda #$80
                sta Status
                rts
		.pend
                
;---------------------------------------
; Timeout handler
;
pclink_timeout	.proc
		dec pclink_timer
		bne vvx
		dec pclink_timer+1
		beq pclink_errexit
vvx             rts
		.pend
;---------------------------------------
; Stop handler
;
pclink_stop	.proc
		jsr check_stop
                bne pclink_errexit.mv

pclink_errexit 	.proc
		#xit_exit
		lda #63
		sta $dd02
		ldy #1
                sec
mv		rts
		.pend
;---------------------------------------
; Save a file
;
;         1            1                2        1   1    X     1   1
;       ___________________________________________     __________     ___
; C64  | "W" | SECONDARY ADDRESS | BYTES LO/HI | 0 |   | DATA | 0 |   | CLOSE
;      |_____|___________________|_____________|___|   |______|___|   |___
;   ___                                             ___            ___
; OPEN |                                           | ? |          | ? |
;   ___|                                           |___|          |___|
;
;         1            1                2         1    1   1    X      1    1   1
;       _________________________________________________     ________________     ___
; C64  | "W" | SECONDARY ADDRESS | BYTES LO/HI | CRC | 0 |   | DATA | CRC | 0 |   | CLOSE
;      |_____|___________________|_____________|_____|___|   |______|_____|___|   |___
;   ___                                                   ___                  ___
; OPEN |                                                 | ? |                | ? |
;   ___|                                                 |___|                |___|
;
; $00 - OK
;
pclink_save	.proc
		#xit_init

		ldx #11
iukq		lda peek_r,x
		sta start_zeropage,x
		dex
		bpl iukq

		lda #$61
                sta pclink_secaddr
		jsr pclink_send_open
		eor #4			;writeonly?
		bne pclink_saveerr
                sta Status

                lda $ae
                sec
                sbc $c1
                tax
                lda $af
                sbc $c2
                bcs mbd
                #lda0
                tax
mbd		cpx #254
		inx
                inx
                adc #0
		pha
                #phx_a

                lda #"W"
                jsr pclink_init
                pla
                jsr pclink_send
                pla
                jsr pclink_send
                jsr pclink_send
                jsr pclink_recieve
		bne pclink_saveerr

		jsr OP_save_saving

		#ldatax $c1
		jsr pclink_send
		lda $c2
		sta peek_r.at1+2
		jsr pclink_send

htj		cpx $ae
		lda peek_r.at1+2
		sbc $af
		bcs xjqxu
		jsr start_zeropage
		inx
		bne htj
                jsr pclink_stop
		inc peek_r.at1+2
		#jmps htj

xjqxu		stx $ac
		lda peek_r.at1+2
		sta $ad
		#lda0
		jsr pclink_send
                jsr pclink_recieve
                jsr pclink_send_close
		tay
		rts
		.pend
;---------------------------------------
; Open a file
;
;         1             1              X       1   1
;       _________________________________________
; C64  | "O" | SECONDARY ADDRESS | FILENAME  | 0 |
;      |_____|___________________|___________|___|
;                                                 ___
;                                                | ? |
;                                                |___|
;
;          1            1              X       1     1    1   1
;       ____________________________________________________
; C64  | !"O" | SECONDARY ADDRESS | FILENAME  | 0 | CRC | 0 |
;      |______|___________________|___________|___|_____|___|
;                                                            ___
;                                                           | ? |
;                                                           |___|
; $00 readonly
; $02 read+write
; $04 writeonly
; $06 error
;
pclink_send_open .proc
		.if PCLINK_USECRC=1
                lda #"O"+128
                .else
		lda #"O"
                .fi
		jsr pclink_init
		ldx #0
exq		lda path_exp,x
		beq keuo
		jsr pclink_send
		.if PCLINK_USECRC=1
                lda path_exp,x
                eor pclink_crc
                tay
                lda crc8,y
                sta pclink_crc
                .fi
		inx
		bne exq
keuo		jsr pclink_send
		.if PCLINK_USECRC=1
		lda pclink_crc
		jsr pclink_send
                jsr pclink_send
		.fi
		#jmps pclink_recieve
		.pend
;---------------------------------------
; Close a file
;
;         1             1          1
;       _____________________________
; C64  | "C" | SECONDARY ADDRESS | 0 |
;      |_____|___________________|___|
;                    
;                    
;                    
;
;          1             1           1    1
;       ____________________________________
; C64  | !"C" | SECONDARY ADDRESS | CRC | 0 |
;      |______|___________________|_____|___|
;
;

pclink_send_close
		.proc
                .if PCLINK_USECRC=1
                lda #"C"+128
                .else
		lda #"C"
                .fi
		jsr pclink_init
                .if PCLINK_USECRC=1
                lda pclink_crc
                jsr pclink_send
                .fi
		#jmps pclink_send
		.pend
;---------------------------------------
; Open a file
;
pclink_open     .proc
		#xit_init
                
                lda ActualSecAddr
                sta pclink_secaddr

		jsr pclink_send_open
                lsr
                tay
                lda modes,y
		cpy #1
                rts
modes		.byte INFO_BLK_READ,INFO_BLK_RW,INFO_BLK_WRITE,INFO_IGNORE
		.pend
;---------------------------------------
; Close a file
;
;
pclink_close    .proc
		#xit_init

                lda ActualSecAddr
                sta pclink_secaddr

		#jmps pclink_send_close
		.pend
;---------------------------------------
; Fill chrin buffer
;
pclink_chrin	.proc
		#xit_init

                #ldatax buffer_currentp_exp
                lda buffer_pclink_secaddr,x
                sta pclink_secaddr

                lda #"R"
                jsr pclink_init
                lda #<2048
                jsr pclink_send
                lda #>2048
                jsr pclink_send
                jsr pclink_send
                jsr pclink_recieve
                cmp #0
                bne vs

		#ldatax buffer_currentp_exp
                lda buffer_start,x
                sta pom2
                .if SUPERCPU=1
                ldy #1
                sty pom3
                dey
                .else
                ldy #0
                .fi
                sty pom1

                jsr pclink_recieve
                sta next_sector
                jsr pclink_recieve
                sta next_sector+1

vv              inc next_sector
		beq vx
vx2		jsr pclink_recieve
                #sta_zp_y pom1
                iny
                bne vv
                inc pom2
                gne vv
vx		inc next_sector+1
		bne vx2
                jsr pclink_recieve
vs		rts
		.pend
;---------------------------------------
; Blockread
;
		.comment
pclink_read	.proc
		#xit_init

                #ldatax buffer_currentp_exp
                lda buffer_pclink_secaddr,x
                sta pclink_secaddr

                lda #"R"
                jsr pclink_init
                lda #<2048
                jsr pclink_send
                lda #>2048
                jsr pclink_send
                jsr pclink_send
                jsr pclink_recieve
                cmp #0
                bne vs

		#ldatax buffer_currentp_exp
                lda buffer_start,x
                sta pom2
                .if SUPERCPU=1
                ldy #1
                sty pom3
                dey
                .else
                ldy #0
                .fi
                sty pom1

                jsr pclink_recieve
                sta next_sector
                jsr pclink_recieve
                sta next_sector+1

vv              inc next_sector
		beq vx
vx2		jsr pclink_recieve
                #sta_zp_y pom1
                iny
                bne vv
                inc pom2
                gne vv
vx		inc next_sector+1
		bne vx2
                jsr pclink_recieve
vs		rts
		.pend
                .endc
